#include <stdlib.h>
#include <uf_defs.h>
#include <uf_modl.h>
#include <uf_drf.h>
#include <uf_layout.h>
#include <uf_view.h>
#include "calcproto.h"

/**********************************************************************
*
* calc_dimension_top 
*
* Create dimensions on the top half of the calculator.  Use the routine
* that extracts edges to obtain the objects for dimensioning.  Edges
* are:
*       For horizontal dimensions
*     edge[0]  Calc. left
*     edge[1]  Calc. right
*     edge[2]  Photocell left
*     edge[3]  Photocell right
*     edge[4]  Buttonhole left
*     edge[5]  Buttonhole right
*       For Vertical dimensions
*     edge[6]  Calc. bottom
*     edge[7]  Calc. top
*     edge[8]  LCD bottom
*     edge[9]  LCD top
*     edge[10]  Photocell bottom
*     edge[11]  Photocell top
*     edge[12]  Button bottom
*     edge[13]  Button top
*
* revisions 
*   rv   date      who   reason for change 
*   00   01NOV95   J_P                   
*   01   01JUL96   J_P   Changed to new API dimension calls
*   02   01SEP97   J_P   Changed to new API preferences calls
*   03   14FEB03   DKF   Added new header file to #include list,
*                        Updated error checking
*
* input 
*  (none)  
*
* output 
*  calc_dimension_top       0 = success, >0 = error 
*  
*  
**********************************************************************/

    int calc_dimension_top()
{
/******************** Routine variable declaration  *******************/
    char
        radsym[7],          /* radius symbol string */
        diasym[7];          /* diameter symbol string */

    int mpi[100],           /* integer drafting parameters */
        epcode[14],         /* startpoint-or-endpoint modifier */
        flag;               /* results from functions */

    tag_t
        edge[14],           /* edges to be dimensioned */
        view_tag,           /* Tag of the view containing dimensions */
        dimension;          /* dimension object */

    double
        mpr[70],            /* floating point drafting parameters */
        origin[3],          /* dimension origin */
        height,             /* height of calculator */
        width;              /* width of calculator */

    UF_DRF_text_t
        dim_text;           /* Struct. for appended text and man. dim */

    UF_DRF_object_t
        dim0,               /* Structure for first object in dim. */
        dim1;               /* Structure for second object in dim. */

/******************** Beginning of Executable code *********************/

/* save the top view to a custom view "top_drf" */
/* then do the dimensioning in this new view    */

   UF_CALL( UF_VIEW_ask_work_view(&view_tag) );
   UF_CALL( UF_VIEW_copy_view(view_tag,"TOP_DRF",&view_tag) );
   UF_CALL( uc6464("","","TOP_DRF") );


/* Read the current settings for drafting/dimensioning */
    UF_CALL( UF_DRF_ask_preferences( mpi, mpr, radsym, diasym ) );

/* Set the linear units to mm and change the character size and number
 * of decimal places.  
 * Call the routine that reset draft/dim settings.
 */
    mpi[3]  = 2;        /* decimal places */
    mpi[13] = 1;        /* linear units */
    mpr[32] = 3.0;      /* character height */
    UF_CALL( UF_DRF_set_preferences( mpi, mpr, radsym, diasym ) );

/* Evaluate the expressions for the calculator height and width.  Check
 * return codes.
 */
    UF_CALL( UF_MODL_eval_exp("Body_height", &height) );
    UF_CALL( UF_MODL_eval_exp("Body_width", &width) );

/* Initialize the structures.  This sets the associativity modifiers
 * to UF_DRF_end_point (which is what we want.  Check return codes!
 * Also initialize the text structure.
 */
    UF_CALL( UF_DRF_init_object_structure( &dim0 ) );
    UF_CALL( UF_DRF_init_object_structure( &dim1 ) );
    dim_text.lines_app_text = 0;
    dim_text.user_dim_text = NULL;
    dim_text.appended_text = NULL;

/* Get the edges for the dimensions.  They are returned in a particular
 * order documented in the comments at the routine start.
 */
    flag = calc_dimension_top_edges(edge,epcode);
    if(flag) return (flag);

/* Obtain the tag of the work view.  It is used in the structures. */
    UF_CALL( UF_VIEW_ask_tag_of_view_name( "", &view_tag ) );
    dim0.object_view_tag = dim1.object_view_tag = view_tag;

/* First dim. is from left edge to right.  Set the location. */
    origin[0] = 0.5 * width;
    origin[1] = height + 6.0*mpr[32];
    origin[2] = 5.0;

/* XXX Create the dimension after setting the structure data for
 * both obect tags and assoc. modifiers.
 */
    dim0.object_assoc_modifier = epcode[0];
    dim0.object_tag = edge[0];
    dim1.object_assoc_modifier = epcode[1];
    dim1.object_tag = edge[1];
    UF_CALL( UF_DRF_create_horizontal_dim( &dim0, &dim1,
             &dim_text, origin, &dimension ) );

/* Next dim is the left edge to photocell left edge. */
    origin[0] = 0.5 * width;
    origin[1] = height + 2.0*mpr[32];
    origin[2] = 5.0;
    dim0.object_assoc_modifier = epcode[0];
    dim0.object_tag = edge[0];
    dim1.object_assoc_modifier = epcode[2];
    dim1.object_tag = edge[2];
    UF_CALL( UF_DRF_create_horizontal_dim( &dim0, &dim1,
             &dim_text, origin, &dimension ) );


/* Next dim is from the photocell left edge to photocell right edge */
    origin[0] = 0.5 * width;
    origin[1] = height + 4.0*mpr[32];
    origin[2] = 10.0;
    dim0.object_assoc_modifier = epcode[3];
    dim0.object_tag = edge[3];
    dim1.object_assoc_modifier = epcode[2];
    dim1.object_tag = edge[2];
    UF_CALL( UF_DRF_create_horizontal_dim( &dim0, &dim1,
             &dim_text, origin, &dimension ) );


/* Next dim is left edge to buttonhole left edge. */
    origin[0] = 0.5 * width;
    origin[1] = -4.0*mpr[32];
    origin[2] = 10.0;
    dim0.object_assoc_modifier = epcode[0];
    dim0.object_tag = edge[0];
    dim1.object_assoc_modifier = epcode[4];
    dim1.object_tag = edge[4];
    UF_CALL( UF_DRF_create_horizontal_dim( &dim0, &dim1,
             &dim_text, origin, &dimension ) );


/* Next dim. is the width of the buttonhole.  */
    origin[0] = 0.5 * width;
    origin[1] = -2.0*mpr[32];
    origin[2] = 10.0;
    dim0.object_assoc_modifier = epcode[5];
    dim0.object_tag = edge[5];
    dim1.object_assoc_modifier = epcode[4];
    dim1.object_tag = edge[4];
    UF_CALL( UF_DRF_create_horizontal_dim( &dim0, &dim1,
             &dim_text, origin, &dimension ) );


/* Create vertical dimensions.  */

/* Create the dimension from the calculator bottom to top.  */
    origin[0] = width + 50.0;
    origin[1] = 0.5*height;
    origin[2] = 5.0;
    dim0.object_assoc_modifier = epcode[6];
    dim0.object_tag = edge[6];
    dim1.object_assoc_modifier = epcode[7];
    dim1.object_tag = edge[7];
    UF_CALL( UF_DRF_create_vertical_dim( &dim0, &dim1,
             &dim_text, origin, &dimension ) );


/* Next dim. is from the photocell top to calc. top.  */
    origin[0] = width + 15.0;
    origin[1] = height + 2.0*mpr[32];
    origin[2] = 5.0;
    dim0.object_assoc_modifier = epcode[11];
    dim0.object_tag = edge[11];
    dim1.object_assoc_modifier = epcode[7];
    dim1.object_tag = edge[7];
    UF_CALL( UF_DRF_create_vertical_dim( &dim0, &dim1,
             &dim_text, origin, &dimension ) );


/* Next dim. is the photocell height */
    origin[0] = width + 50.0;
    origin[1] = height + 4.0*mpr[32];
    origin[2] = 5.0;
    dim0.object_assoc_modifier = epcode[11];
    dim0.object_tag = edge[11];
    dim1.object_assoc_modifier = epcode[10];
    dim1.object_tag = edge[10];
    UF_CALL( UF_DRF_create_vertical_dim( &dim0, &dim1,
             &dim_text, origin, &dimension ) );


/* Next dim. is the distance from photocell bottom to LCD top.  */
    origin[0] = width + 15.0;
    origin[1] = 0.5*height;
    origin[2] = 5.0;
    dim0.object_assoc_modifier = epcode[9];
    dim0.object_tag = edge[9];
    dim1.object_assoc_modifier = epcode[10];
    dim1.object_tag = edge[10];
    UF_CALL( UF_DRF_create_vertical_dim( &dim0, &dim1,
             &dim_text, origin, &dimension ) );


/* Next dim. is the height of the LCD.  */
    origin[0] = width + 35.0;
    origin[1] = 0.5*height;
    origin[2] = 5.0;
    dim0.object_assoc_modifier = epcode[9];
    dim0.object_tag = edge[9];
    dim1.object_assoc_modifier = epcode[8];
    dim1.object_tag = edge[8];
    UF_CALL( UF_DRF_create_vertical_dim( &dim0, &dim1,
             &dim_text, origin, &dimension ) );


/* Next dim. is from the calc. bottom to the buttonhole bottom.  */
    origin[0] = -25.0;
    origin[1] = 0.5*height;
    origin[2] = 5.0;
    dim0.object_assoc_modifier = epcode[6];
    dim0.object_tag = edge[6];
    dim1.object_assoc_modifier = epcode[12];
    dim1.object_tag = edge[12];
    UF_CALL( UF_DRF_create_vertical_dim( &dim0, &dim1,
             &dim_text, origin, &dimension ) );


/* The final dimension is the height of the buttonhole.  */
    origin[0] = -60.0;
    origin[1] = 0.5*height;
    origin[2] = 5.0;
    dim0.object_assoc_modifier = epcode[13];
    dim0.object_tag = edge[13];
    dim1.object_assoc_modifier = epcode[12];
    dim1.object_tag = edge[12];
    UF_CALL( UF_DRF_create_vertical_dim( &dim0, &dim1,
             &dim_text, origin, &dimension ) );

    return (0);

}
